
#include <utility.h>
#include <userint.h>

/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*=============== R3267/73 Spectrum Analyzer OPT63(GSM/DECT)=================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: September/20/2000										 */
/* By: Ai Onishi,    Supervisor: Yohei Hirakoso								 */
/*                   SPSpj Gyoda R&D,  ADVANTEST CORPORATION				 */
/*                   Home Page; http://www.advantest.co.jp					 */
/*                   e-mail;    info@advantest.co.jp						 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Version 1.0 :  September/20/2000									 */
/*																			 */
/*===========================================================================*/
 


#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avopt63.h"

#define avopt63_REVISION    "Rev 1.0, 09/00, CVI 5.0.1" /* Instrument driver revision */
#define BUFFER_SIZE         1024L         				/* File I/O buffer size */
    

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/


/****
 ****	Opt63 Static Command Arr
 ****/

/* GSM Standard */
static ViString modSysGsmArr[]={"8PSK","GMSK"};
static ViString modTypGsmArr[]={"GSM450","GSM480","GSM850","GSM900","DCS1800","PCS1900"};
static ViString measMdGsmArr[]={"BURST","MBURST","CONT"};
static ViString linkGsmArr[]={"MS","BTS"};
static ViString bTypGsmArr[]={"B88","B148"};
static ViString mfltMdGsmArr[]={"NARW","WIDE"};
static ViString pwrCtrGsmArr[]={"16","17","18","19","11","12","13","14","15","99"};
static ViString freqInptGsmArr[]={"CHL","FREQ"};
static ViString inputGsmArr[]={"RF","IQ"};
static ViString bbInputGsmArr[]={"AC","DC"};
static ViString iqModeGsmArr[]={"NORM","INV"};

/* Dect Standerd */
static ViString measMdDectArr[]={"BURST","CONT"};
static ViString linkDectArr[]={"PP","RFP"};
static ViString phypktDectArr[]={"P00","P32","P08","P80"};
static ViString pwrLvlDectArr[]={"2","1"};
static ViString syncTypeDectArr[]={"NO","SYNC"};

/* On/Off Arr */
static ViString switchArr[] = {"OFF","ON"};

/* Edit Channel */
static ViString edtChArr[]={"UP1","DN1","UP2","DN2","UP3","DN3"};

/* Channel Enable */
static ViString enableArr[]={"DSBL","ENBL"};

/* Y Scale Sel */
static ViString YscaleModeArr[] ={"TDP","OOR","FDP"};
static ViString YscaleDivArr[] ={"P10DB","P5DB","P2DB"};

/* Trigger */
static ViString trgWFSrcArr[] = {"TRGSRC FREE","TRGSRC VIDEO","TRGSRC IF","TRGSRC EXT"};
static ViString trgSlopeArr[] = {"TRGSLP FALL","TRGSLP RISE"};

/* Template Edit */
static ViString UpLowArr[]={"LOW","UP"};

/* Preselecter */
static ViString preselArr[]={"36G","16G"};

/* Modulation GSM */
static ViString gsmTrgArr[]={"FREE","IF","EXT"};
static ViString slopeArr[]={"FALL","RISE"};
static ViString tdmaStrArr[]={"TYP2","TYP1"};
static ViString pwrVsTimeMmArr[]={"HIGH","NORM"};
static ViString yScaleGSMArr[]={"P20DB","P10DB","P5DB"};

/* DECT Parameter */
static ViString dfmBitSeqArr[]={"STD","RND"};

/* Pwr vs Time */
static ViString  zeroOneArr[] ={"1","0"};

/* Avarage */
static ViString avgFunctionArr[] = {"TDPAVG", "OORAVG", "TDSAVG", "FDPAVG", "OBWAVG",
									"DTSAVG", "DTMAVG", "SPRAVG","FDSAVG"};
static ViString avgFuncGsmArr[] = {"TAVGPH","TAVGMOD","TXAVG","GPTAVG"};
static ViString avgFuncDectArr[] = {"DFMAVG", "DTJAVG", "TXAVG", "PTAVG "};

/* Auto Level */
static ViString autoLvlFunctionArr[] = {"TDPAUTOLVL","OORAUTOLVL","TDSAUTOLVL",
										"FDPAUTOLVL","OBWAUTOLVL","DTSAUTOLVL",
										"DTMAUTOLVL","SPRAUTOLVL","FDSAUTOLVL",
										"AUTOLVL"};

/* Mesurement Start */
static ViString funcMeasureArr[] = {"TDPMEAS","OORMEAS","TDSMEAS","FDPMEAS", "OBWMEAS", "DTSMEAS",
									"DTMMEAS","CINBSPR","FDSMEAS","SI"};

static ViString funcMeasGsmArr[] = {"PHACC","MODACC","TXPWR","PWRTM","SI"};

static ViString funcMeasDectArr[] = {"DFMDEV","DTJIT","TXPWR","PWRTM","SI"};


/* Graph Type */
static ViString gTypPEgsmArr[] = {"GGPTYP INP","GGPTYP LIN","GGPTYP DOT",
			"GGPTYP CON","GGPTYP IEYE","GGPTYP QEYE","GGPTYP IQEYE","GGPTYP TR",
			"GGPTYP PE","GGPTYP FPE","GGPTYP BF","GGPTYP FREYE","GGPTYP DE"};
			
			
static ViString gTypGsmArr[] = {"EGPTYP INP",
			"EGPTYP LIN","EGPTYP DOT","EGPTYP CON",
			"EGPTYP IEYE","EGPTYP QEYE","EGPTYP IQEYE",
			"EGPTYP DEMOD","EGPTYP EVM","EGPTYP ME","EGPTYP PFE",
			"EGPTYP 8INP","EGPTYP 8LIN","EGPTYP 8DOT","EGPTYP 8CON",
			"EGPTYP 8IEYE","EGPTYP 8QEYE","EGPTYP 8IQEYE"};

static ViString gTypDectArr[] = {"DFMGTYP BIT","DFMGTYP EYE","DFMGTYP DEMOD"};


/* Gate */
static ViString srcTrgArr[] = {"SRC EXT", "SRC TRG"};

static ViString trgFdpSrcArr[] ={"FREE","VIDEO","IF","EXT"};

/* Detecter Select */
static ViString DetArr[]={"DET NRM","DET POS","DET NEG","DET SMP"};

/* Select Table 1 to 3 */
static ViString selTblArr[] = {"TBL 1","TBL 2","TBL 3"};

/* Table Frequency Type */   
static ViString tableFArr[] = {"TBLF REL","TBLF ABS"};

/*TDS Param*/
static ViString resultArr[] = {"RES RMS","RES PK"};

/* DTS Param */
static ViString FreqSetArr[] ={"FRMD CFSP","FRMD STSP"};
static ViString LvlUnitArr[] ={"UNIT DBM", "UNIT W", "UNIT DBUV"};
static ViString dtsResultArr[] = {"RES MKR", "RES REL","RES ABS"};
static ViString refMkrArr[] = {"REF MOD","REF MKR"};

/* Tx Power */
static ViString TXtrigSourceArr[] = {"TTRGSRC INT","TTRGSRC EXT","TTRGSRC INTRVL1"};
static ViString measFltArr[] = {"NARW","WIDE"};

/* Waveform quality */
static ViString wfqGraphArr[] = {"CON","CONDOT", "ICHEYE", "QCHEYE", "MAGERR", "PHAERR", "EVM"};
static ViString wfqGraphYdataArr[] = {"GPHQ?","GPHQ?","GPHI?","GPHQ?","GPHY?","GPHY?","GPHY?"};
static ViString wfqGraphMkrXdataArr[] = {"GMKQ?","GMKQ?", "GMKI?", "GMKQ?","GMKERR?","GMKDEG?","GMKERR?"};



/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avopt63_stringValPair is used in the avopt63_errorMessage function        */
/* avopt63_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  avopt63_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  avopt63_tStringValPair;
 
struct avopt63_statusDataRanges {
    ViInt16 traceRange;
    ViInt16 harmonics;
    ViChar instrDriverRevision[256];
};

typedef struct avopt63_statusDataRanges *avopt63_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avopt63_invalidViBooleanRange (ViBoolean val);
ViBoolean avopt63_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avopt63_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avopt63_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avopt63_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avopt63_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avopt63_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avopt63_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus avopt63_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn,
	ViUInt32 tmoIn, ViPUInt16 STB);
ViStatus avopt63_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
	ViStatus currentStatus);
ViStatus avopt63_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Standard Setting GSM                                            */
/* Purpose:  This function sets the communication type and link of the signal*/
/*           under TRANSIENT measurement.									 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_std_setGSM (ViSession instrumentHandle,
                                               ViBoolean modulation,
                                               ViInt16 type, ViInt16 measMode,
                                               ViBoolean link,
                                               ViBoolean burstType,
                                               ViBoolean syncType, ViInt16 TSC,
                                               ViBoolean fillterMode,
                                               ViInt16 powerCtrLvl,
                                               ViReal64 offsetLevel,
                                               ViBoolean frequencyInput,
                                               ViBoolean input,
                                               ViBoolean basebandInput,
                                               ViBoolean IQInverse,
                                               ViBoolean contAutoLevelSet)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE],SyncBuf[20];
    ViStatus avopt63_status = VI_SUCCESS;
    
    
	if (avopt63_invalidViBooleanRange (modulation))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViInt16Range (type, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViInt16Range (measMode, 0, 2))
        return VI_ERROR_PARAMETER4;    
    if (avopt63_invalidViBooleanRange (link))
        return VI_ERROR_PARAMETER5;
    if (avopt63_invalidViBooleanRange (burstType))
        return VI_ERROR_PARAMETER6;
    if (avopt63_invalidViBooleanRange (syncType))
        return VI_ERROR_PARAMETER7;    
    if (avopt63_invalidViInt16Range (TSC, 0, 7))
        return VI_ERROR_PARAMETER8;
    if (avopt63_invalidViBooleanRange (fillterMode))
        return VI_ERROR_INSTR_PARAMETER9;         
    if (avopt63_invalidViInt16Range (powerCtrLvl, 0, 9))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt63_invalidViReal64Range (offsetLevel, -100.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER11;
    if (avopt63_invalidViBooleanRange (frequencyInput))
        return VI_ERROR_INSTR_PARAMETER12;
    if (avopt63_invalidViBooleanRange (input))
        return VI_ERROR_INSTR_PARAMETER13;
    if (avopt63_invalidViBooleanRange (basebandInput))
        return VI_ERROR_INSTR_PARAMETER14;
    if (avopt63_invalidViBooleanRange (IQInverse))
        return VI_ERROR_INSTR_PARAMETER15;
    if (avopt63_invalidViBooleanRange (contAutoLevelSet))
        return VI_ERROR_INSTR_PARAMETER16;        

	if ( syncType == 1)
	{
		if ( burstType == 1)
		{
			Fmt (SyncBuf, "%s<TSC%d",TSC);      	
		}
		else
		{
			Fmt (SyncBuf, "%s<BIT");
		}
	}
	else
	{
		Fmt (SyncBuf, "%s<NO");
	}
		
	Fmt (wrtBuf, "%s<MODSYS %s;MODTYP %s;MEASMD %s;LINK %s;BTYP %s;SYNC %s;MFLTMD %s;PWCTL %s;RO %fDB;FINPMD %s;INPUT %s;BBINPUT %s;IQMD %s;ALS %s",
		modSysGsmArr[modulation],
		modTypGsmArr[type],
		measMdGsmArr[measMode],
		linkGsmArr[link],
		bTypGsmArr[burstType],
		SyncBuf,
		mfltMdGsmArr[fillterMode],
		pwrCtrGsmArr[powerCtrLvl],
		offsetLevel,
		freqInptGsmArr[frequencyInput],
		inputGsmArr[input],
		bbInputGsmArr[basebandInput],
		iqModeGsmArr[IQInverse],
		switchArr[contAutoLevelSet]);
		
    total += NumFmtdBytes();
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: Standard Setting DECT                                           */
/* Purpose:  This function sets the communication type and link of the signal*/
/*           under TRANSIENT measurement.									 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_std_setDECT (ViSession instrumentHandle,
                                                ViBoolean measMode,
                                                ViBoolean link,
                                                ViInt16 physicalPacket,
                                                ViBoolean ZFiled,
                                                ViBoolean syncType,
                                                ViBoolean fillterMode,
                                                ViBoolean powerLevel,
                                                ViReal64 offsetLevel,
                                                ViBoolean frequencyInput,
                                                ViBoolean input,
                                                ViBoolean basebandInput,
                                                ViBoolean IQInverse,
                                                ViBoolean contAutoLevelSet)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    
	if (avopt63_invalidViBooleanRange(measMode))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange(link))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViInt16Range(physicalPacket, 0, 3))
        return VI_ERROR_PARAMETER4;        
    if (avopt63_invalidViBooleanRange(ZFiled))
        return VI_ERROR_PARAMETER5;
    if (avopt63_invalidViBooleanRange(syncType))
        return VI_ERROR_PARAMETER6;    
    if (avopt63_invalidViBooleanRange(fillterMode))
        return VI_ERROR_PARAMETER7;         
    if (avopt63_invalidViBooleanRange(powerLevel))
        return VI_ERROR_PARAMETER8;
    if (avopt63_invalidViReal64Range (offsetLevel, -100.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt63_invalidViBooleanRange (frequencyInput))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt63_invalidViBooleanRange (input))
        return VI_ERROR_INSTR_PARAMETER11;
    if (avopt63_invalidViBooleanRange (basebandInput))
        return VI_ERROR_INSTR_PARAMETER12;
    if (avopt63_invalidViBooleanRange (IQInverse))
        return VI_ERROR_INSTR_PARAMETER13;
    if (avopt63_invalidViBooleanRange (contAutoLevelSet))
        return VI_ERROR_INSTR_PARAMETER14;        

	Fmt (wrtBuf, "%s<MEASMD %s;LINK %s;PHYPKT %s;ZFILD %s;SYNC %s;MFLTMD %s;PWRLVL %s;RO %fDB;FINPMD %s;INPUT %s;BBINPUT %s;IQMD %s;ALS %s",
		measMdDectArr[measMode],
		linkDectArr[link],
		phypktDectArr[physicalPacket],
		switchArr[ZFiled],
		syncTypeDectArr[syncType],
		mfltMdGsmArr[fillterMode],
		pwrLvlDectArr[powerLevel],
		offsetLevel,
		freqInptGsmArr[frequencyInput],
		inputGsmArr[input],
		bbInputGsmArr[basebandInput],
		iqModeGsmArr[IQInverse],
		switchArr[contAutoLevelSet]);
		
    total += NumFmtdBytes();
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;     
}



/*===========================================================================*/
/* Function: Channel Setting                                            	 */
/* Purpose: This function sets the Channel Number.                           */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_number_setChannel (ViSession instrumentHandle,
                                            ViInt16 channelNo)
{
    ViUInt32 retCnt = 0;

    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
 
    if (avopt63_invalidViInt16Range(channelNo, -12800, 12800))
        return VI_ERROR_PARAMETER2; 
        
    Fmt (wrtBuf, "%s<CH %d",channelNo);
        
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;      
}



/*===========================================================================*/
/* Function: Channel Edit 		                                             */
/* Purpose: This function edits channel setting.                             */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViInt16 editChannel, ViInt16 startCHNum,
                                ViInt16 stopCHNum, ViReal64 baseFreqHz,
                                ViReal64 channelSpaceHz, ViInt16 CHOffset)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViInt16Range (editChannel, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViInt16Range (startCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViInt16Range (stopCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER5;
    if (avopt63_invalidViReal64Range (baseFreqHz, 0.0, 100000000000.0))
        return VI_ERROR_PARAMETER6;
    if (avopt63_invalidViReal64Range (channelSpaceHz, -100000000.0, 100000000.0))
        return VI_ERROR_PARAMETER7;
    if (avopt63_invalidViInt16Range (CHOffset, -12800, 12800))
        return VI_ERROR_PARAMETER8;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<CHSETSTD");
    	total += NumFmtdBytes();
	}
	else
	{
		Fmt (wrtBuf, "%s<CHED%s %d,%d,%fHZ,%fHZ,%d", edtChArr[editChannel],
    	startCHNum,stopCHNum,baseFreqHz,channelSpaceHz,CHOffset);
    	total += NumFmtdBytes();
	}

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;  
   
}


/*===========================================================================*/
/* Function: Channel Enable ON/OFF                                           */
/* Purpose:  This function sets the state of Channel table enable.           */
/*                                      									 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViBooleanRange (channel1))
        return VI_ERROR_PARAMETER2;	
    if (avopt63_invalidViBooleanRange (channel2))
        return VI_ERROR_PARAMETER3;	   
    if (avopt63_invalidViBooleanRange (channel3))
        return VI_ERROR_PARAMETER4;	           

	Fmt (wrtBuf, "%s<CHTBL1 %s;CHTBL2 %s;CHTBL3 %s",
		enableArr[channel1],
		enableArr[channel2],
		enableArr[channel3]);
    total += NumFmtdBytes();


    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;      
}
/*===========================================================================*/
/* Function: Y Scale Setup			                                         */
/* Purpose:  This function sets the state of Channel table enable.           */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_Yscale_setup (ViSession instrumentHandle,
                                       ViInt16 measMode, ViInt16 division)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (measMode, 0, 2))
        return VI_ERROR_PARAMETER2;     
    if (avopt63_invalidViInt16Range (division, 0, 2))
        return VI_ERROR_PARAMETER3;
        
	Fmt (wrtBuf, "%s<%sDIV %s",
		YscaleModeArr[measMode],
		YscaleDivArr[division]);
    total += NumFmtdBytes();


    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;      
}    


/*===========================================================================*/
/* Function: TDP Trigger Conditions                                          */
/* Purpose:  This function is used for time domain Power. This               */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, 
                                        ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt63_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<%s;%s;TRGLVL %dENT;TRGPOS %dENT;TRGDT %fSC", 
    	trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope], 
    	triggerLevel, 
    	triggerPosition, 
    	delayTimes);
    total += NumFmtdBytes();
 
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;       
    
}

/*===========================================================================*/
/* Function: TDP Window Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Window setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViReal64Range (winPosS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt63_invalidViReal64Range (winWidS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
        Fmt (wrtBuf, "%s<TDPWDO %s; TDPWPOS %fSC; TDPWWID %fSC",
        	switchArr[window],
    		winPosS, winWidS);
    	total += NumFmtdBytes();
    }
               
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}


/*===========================================================================*/
/* Function: TDP Template Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Template setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_template_SetTDP (ViSession instrumentHandle,
                                          ViBoolean template, ViReal64 shiftXS,
                                          ViReal64 shiftYdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

	if (avopt63_invalidViBooleanRange (template))
        return VI_ERROR_PARAMETER2;

    if (avopt63_invalidViReal64Range (shiftXS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER3;   
    if (avopt63_invalidViReal64Range (shiftYdB, -200.0, 100.0))
        return VI_ERROR_PARAMETER4; 

    Fmt (wrtBuf, "%s<TDPTMPL %s; TDPTMPLSX %fSC; TDPTMPLSY %fDB", 
    	switchArr[template],
    	shiftXS, 
    	shiftYdB);
    
    total += NumFmtdBytes();
    
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;      
}


/*===========================================================================*/
/* Function: TDP Template Edit					                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the Table data of Template.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

	if (avopt63_invalidViBooleanRange (templateSelect))
        return VI_ERROR_PARAMETER3;
	if (avopt63_invalidViInt16Range (operation,0,2 ))
        return VI_ERROR_PARAMETER2;
 
 
    Fmt (wrtBuf, "%s<TDPTMPLSEL %s", UpLowArr[templateSelect]);
	total += NumFmtdBytes();



	switch (operation)
	{
	case 0: /* copy from STD */
		Fmt (wrtBuf, "%s[a]<; TDPTMPLCP");
		break;
		
	case 1: /* entry */
		if (avopt63_invalidViReal64Range (templateTimes, -1.0, 1.0))
        	return VI_ERROR_PARAMETER4;
/*		if (avopt63_invalidViReal64Range (templateLevel, -200.0, 100.0))
        	return VI_ERROR_PARAMETER5; 
*/
		Fmt (wrtBuf, "%s[a]<; TDPTMPLED %fSC,%fDB", templateTimes, templateLevel);
		break;
	case 2: /* Table init */
		 Fmt(wrtBuf, "%s[a]<; TDPTMPLCLR"); 
		break;
	}
		total += NumFmtdBytes(); 
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: TDP Parameter Setup				                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    }
    else
    {
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
    	if (avopt63_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5;
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        

        Fmt (wrtBuf, "%s<TDP%s; TDP%s; TDPTMPLPW %s; TDPTMPLBTM %fDB; TDPJDG %s; TDPJDGUP %fDB; TDPJDGLOW %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[TMPLCoupleofPWR],
        	templateLimit, switchArr[judgement], upperLimit,  lowerLimit);

    }
    	total += NumFmtdBytes();                 
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;        
}


/*===========================================================================*/
/* Function: On/Off Raito Trigger   				                         */
/* Purpose:  This function is used for On/Off Raito (Time Domain).			 */
/*           This function sets the parameters for Trigger. 				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt63_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<OOR%s; OOR%s; OORTRGLVL %dENT",
    	trgWFSrcArr[triggerSource], trgSlopeArr[trgSlope], triggerLevel);
    total += NumFmtdBytes();
    
	if ((triggerSource == 1) || (triggerSource == 2)) {
	    Fmt (wrtBuf, "%s[a]<;OORTRGPOS %dENT; OORTRGDT %fSC", triggerPosition, delayTimes);
    	total += NumFmtdBytes(); 
	}

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;       
    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Window						                     */
/* Purpose:  This function is used for time domain wave form analyzing. This */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViReal64Range (onPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt63_invalidViReal64Range (onWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 
    if (avopt63_invalidViReal64Range (offPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER6;   
    if (avopt63_invalidViReal64Range (offWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER7; 
        
    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    }
    else
    {
        Fmt (wrtBuf, "%s<OORWDO %s; OORWONPOS %fSC; OORWONWID %fSC; OORWOFPOS %fSC; OORWOFWID %fSC",
        	switchArr[window], onPositionS, onWidthS, offPositionS, offWidthS);
    }
    	total += NumFmtdBytes();           
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Parameter Setup				                     */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
    
    
        Fmt (wrtBuf, "%s<OOR%s; OOR%s; OORJDG %s; OORJDGUP %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[judgement], upperLimit);
    	total += NumFmtdBytes();
    }
               
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}
/*===========================================================================*/
/* Function: TDS Trigger Set	                                             */
/* Purpose:  This function is used for time domain Spurious. This            */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;    
    if (avopt63_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<TDS%s;TDS%s", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope]);
    total += NumFmtdBytes();
    
    
	if (triggerSource != 0 ) {
		
		if ((triggerSource == 1) || (triggerSource == 2)) {
	    	Fmt (wrtBuf, "%s[a]<;TDSTRGLVL %dENT", triggerLevel);
	    	total += NumFmtdBytes();		
		}
		
	    Fmt (wrtBuf, "%s[a]<;TDSTRGPOS %dENT;TDSTRGDT %fSC", triggerPosition, delayTimes);
	    total += NumFmtdBytes();     
	}

    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;       
}

/*===========================================================================*/
/* Function: TDS Table Edit						                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the Table for Measurement.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level)
{
    ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
	if (avopt63_invalidViInt16Range (operation, 0, 6))
        return VI_ERROR_PARAMETER2;    
	if (avopt63_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

/*

	0: Load Table
	1: Save Table
	2: Select Table (Default value)
	3: Edit Table
	4: Delete Table
	5: Init Table
	6: Table Freq Type & Carrer Freq Input

*/

	switch (operation)
	{
	case 0: /* LOAD TABLE */
		Fmt (wrtBuf, "%s<RCL%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 1: /* SAVE TABLE */
		Fmt (wrtBuf, "%s<SVS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 2: /* SELECT TABLE ONLY */
		Fmt (wrtBuf, "%s<TDS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 3: /* DATA ENTRY */
		Fmt (wrtBuf, "%s<TDSTBLED %fHZ,%fDB",frequency, level);
    	total += NumFmtdBytes();
		break;
	case 4: /* DATA DELETE */
		Fmt (wrtBuf, "%s<TDSTBLDEL");
    	total += NumFmtdBytes();
		break;
	case 5: /* INIT TABLE */
		Fmt (wrtBuf, "%s<TDSCLR");
    	total += NumFmtdBytes();
		break;
	case 6: /* TABLE FREQ & CARRER FREQ */
		if (avopt63_invalidViBooleanRange (freqType))
        	return VI_ERROR_PARAMETER4;
        if (avopt63_invalidViReal64Range (carrierFreq, 0.0, 8000000000.0))
        	return VI_ERROR_PARAMETER5;	
		Fmt (wrtBuf, "%s<TDS%s; CF %fHZ",tableFArr[freqType],carrierFreq);
		break;
	}
	total += NumFmtdBytes();
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        	return avopt63_status;    
    return avopt63_status;     
}

/*===========================================================================*/
/* Function: TDS Parameter Setup				                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViBoolean result, ViReal64 peakMKRYDelta,
                                       ViReal64 multiplier,
                                       ViBoolean preselector)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDSSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
	    if (avopt63_invalidViBooleanRange (result))
        	return VI_ERROR_PARAMETER6;    
   		if (avopt63_invalidViReal64Range (peakMKRYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER7;
        if (avopt63_invalidViReal64Range (multiplier, 1.0, 100.0))
        	return VI_ERROR_PARAMETER8;
        if (avopt63_invalidViBooleanRange (preselector))
        	return VI_ERROR_INSTR_PARAMETER9; 	
         
        Fmt (wrtBuf, "%s<TDS%s; TDS%s; TDSJDG %s; TDS%s; TDSPKMKY %fENT; TDSMULTI %fENT;TDSPRE %s",
        	DetArr[detector], 
        	LvlUnitArr[displayUnit],
        	switchArr[judgement], 
        	resultArr[(ViInt16)result],
        	peakMKRYDelta, multiplier,preselArr[preselector]);

    	total += NumFmtdBytes();
    }
              
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    	    
}

/*===========================================================================*/
/* Function: Gated Spectrum	Trigger 			                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Setup Trigger parameters.	     		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_trigger_setGtSpc (ViSession instrumentHandle,
                                           ViInt16 trgSource, ViBoolean trgSlope,
                                           ViInt16 triggerLevel,
                                           ViInt16 triggerPosition,
                                           ViReal64 delayTimes,
                                           ViBoolean gateSource,
                                           ViBoolean gateSettoSTD,
                                           ViReal64 gatePositions,
                                           ViReal64 gateWidths,
                                           ViBoolean gatedSweep,
                                           ViInt16 detector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	if (avopt63_invalidViInt16Range (trgSource,0,3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt63_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;
    if (avopt63_invalidViBooleanRange (gateSource))
        return VI_ERROR_PARAMETER7;
    if (avopt63_invalidViBooleanRange (gateSettoSTD))
        return VI_ERROR_PARAMETER8;
    if (avopt63_invalidViReal64Range (gatePositions, 0, 1))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt63_invalidViReal64Range (gateWidths, 0, 1))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt63_invalidViBooleanRange (gatedSweep))
        return VI_ERROR_INSTR_PARAMETER11;    
    if (avopt63_invalidViInt16Range (detector, 0, 3))
        return VI_ERROR_INSTR_PARAMETER12;

    if ( gateSettoSTD == 1)
   	   {
   	    	Fmt (wrtBuf, "%s<TGTTRG %s; TGT%s; TGTTRGLVL %d; TGTTRGPOS %d; TGTTRGDT %fSC; TGT%s; FDPSETSTD; TGTSWP %s; TGT%s; ",
    			trgFdpSrcArr[trgSource],
    			trgSlopeArr[trgSlope], 
    			triggerLevel, triggerPosition, delayTimes, 
    			srcTrgArr[gateSource],
    			switchArr[gatedSweep],DetArr[detector]);
   	   }
   	   else
   	   {
   	   		Fmt (wrtBuf, "%s<TGTTRG %s; TGT%s; TGTTRGLVL %d; TGTTRGPOS %d; TGTTRGDT %fSC; TGT%s; TGTPOS %fSC; TGTWID %fSC; TGTSWP %s; TGT%s",
    			trgFdpSrcArr[trgSource],
    			trgSlopeArr[trgSlope], 
    			triggerLevel, triggerPosition, delayTimes, 
    			srcTrgArr[gateSource],
    			gatePositions,gateWidths, switchArr[gatedSweep],DetArr[detector]);
   	   }

    total += NumFmtdBytes();
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: FDP Window Set						                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	}
	else
	{
	    if (avopt63_invalidViBooleanRange (window))
        	return VI_ERROR_PARAMETER3;
	    if (avopt63_invalidViReal64Range (positionHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER4;
    	if (avopt63_invalidViReal64Range (widthHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER5;
        	
		Fmt (wrtBuf, "%s<FDPWDO %s;FDPWPOS %fHZ; FDPWWID %fHZ", 
			switchArr[window], positionHz, widthHz);
	}
        
        
    total += NumFmtdBytes();
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;

}

/*===========================================================================*/
/* Function: FDP Parameter Set					                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	    total += NumFmtdBytes();
	}
	else
	{
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
		if (avopt63_invalidViBooleanRange (gatedSweep))
        	return VI_ERROR_PARAMETER5;     	
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        

        Fmt (wrtBuf, "%s<FDP%s; FDP%s;TGTSWP %s; FDPJDG %s; FDPJDGUP %f; FDPJDGLOW %f",
        	DetArr[detector], LvlUnitArr[displayUnit],
        	switchArr[gatedSweep], switchArr[judgement],
        	upperLimit,  lowerLimit);
    	total += NumFmtdBytes();	
	}
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;	
}

/*===========================================================================*/
/* Function: OBW Parameter Set                                               */
/* Purpose:  This function sets the parameters for OBW measurement.			 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<OBWSETSTD");	
	    total += NumFmtdBytes();    
	
	}
	else
	{
	    if (avopt63_invalidViReal64Range (OBW, 0.5, 99.5))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
	    if (avopt63_invalidViReal64Range (upperLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER6;
	    if (avopt63_invalidViReal64Range (lowerLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER7;

        Fmt (wrtBuf, "%s<OBWPER %f; OBW%s; OBWJDG %s; OBWJDGUP %fHZ; OBWJDGLOW %fHZ",
        	OBW, DetArr[detector],
			switchArr[judgement], upperLimit, lowerLimit);
    	total += NumFmtdBytes();	
	}
        
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;		
}


/*===========================================================================*/
/* Function: DT Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTSSETSTD");	
	    total += NumFmtdBytes();
	}
	else
	{
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
   		if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt63_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5; 
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER7;  
   		if (avopt63_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER8;         	
    	if (avopt63_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_INSTR_PARAMETER9;
   		if (avopt63_invalidViBooleanRange (refPower))
        	return VI_ERROR_INSTR_PARAMETER10;
        if (avopt63_invalidViReal64Range (symbolRate1T, 1, 100e+9))
        	return VI_ERROR_INSTR_PARAMETER11;
		if (avopt63_invalidViReal64Range (rolloffFactor, 0.01, 0.99))
        	return VI_ERROR_INSTR_PARAMETER12;

        Fmt (wrtBuf, "%s<DTS%s; DTS%s; DTS%s; DTS%s; DTS%s;DTSTMPLPW %s;DTSTMPLBTM %fDB; DTSJDG %s; DTSSYMRT %fHZ; DTSRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit], switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement], symbolRate1T, rolloffFactor);

    	total += NumFmtdBytes();	
	}
        
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;	    
}

/*===========================================================================*/
/* Function: DT Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

    if (avopt63_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt63_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt63_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTSTMPL %s; DTSTMPLSX %fHZ; DTSTMPLSY %fDB; DTSTMPLDX %fHZ",
    	switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;       
}



/*===========================================================================*/
/* Function: DT Template STOP                                                */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB)
{
  	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;   

    if (avopt63_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTSTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt63_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<DTSTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTSTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;
}

/*===========================================================================*/
/* Function: DT Marker Edit                                                  */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

	if (avopt63_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTSMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt63_invalidViBooleanRange(offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt63_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt63_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt63_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
           	Fmt (wrtBuf, "%s<DTSMKRED %d,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);
			
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTSMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt63_invalidViInt16Range(refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt63_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;           	

			Fmt (wrtBuf, "%s<DTSMKRCLR; DTSMKRED %dENT,0HZ,%fHZ,0DB",
				refMarkerType,refBandWidthHz);

			break;
	}
	total += NumFmtdBytes();
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;      
}

/*===========================================================================*/
/* Function: DM Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTMSETSTD");	
	   total += NumFmtdBytes();            
	}
	else
	{
   		if (avopt63_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
    	if (avopt63_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER5;
  		if (avopt63_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER7;
   		if (avopt63_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER8; 
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER9;  
 		if (avopt63_invalidViReal64Range(symbolRate1T,1,1e9))
        	return VI_ERROR_INSTR_PARAMETER10; 
		if (avopt63_invalidViReal64Range(rolloffFactor,0.01,0.99))
        	return VI_ERROR_INSTR_PARAMETER11;
 
        Fmt (wrtBuf, "%s<DTM%s; DTM%s; DTM%s; DTM%s; DTM%s; DTMTMPLPW %s; DTMTMPLBTM %fDB; DTMJDG %s;DTMSYMRT %fHZ; DTMRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement],symbolRate1T,rolloffFactor);
    	total += NumFmtdBytes();	
	}
        
 
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;	    	    
}

/*===========================================================================*/
/* Function: DM Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

    if (avopt63_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt63_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt63_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTMTMPL %s; DTMTMPLSX %fHZ; DTMTMPLSY %fDB; DTMTMPLDX %fHZ",
    	switchArr[template], shiftXHz, shiftYdB, deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;           
}

/*===========================================================================*/
/* Function: DM Template Edit                                                */
/* Purpose:  This function edets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;   

    if (avopt63_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTMTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt63_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<DTMTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTMTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;   
}

/*===========================================================================*/
/* Function: DM Marker Table Edit  											 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViBoolean refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

	if (avopt63_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTMMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt63_invalidViBooleanRange (offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt63_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt63_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt63_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
			Fmt (wrtBuf, "%s<DTMMKRED %d,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);

			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTMMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt63_invalidViInt16Range (refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt63_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4; 
			Fmt (wrtBuf, "%s<DTMMKRCLR; DTMMKRED %dENT,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	total += NumFmtdBytes();
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;          
}


/*===========================================================================*/
/* Function: InBand Spurious Parameter Set                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/


ViStatus CVIFUNC avopt63_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<SPRSETSTD");	
	}
	else
	{
   		if (avopt63_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
  		if (avopt63_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER5; 
    	if (avopt63_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt63_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER7;        	
   		if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER8;
   		if (avopt63_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_INSTR_PARAMETER9; 
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER11; 

        Fmt (wrtBuf, "%s<SPR%s; SPR%s;SPRPKMKY %fENT; SPR%s; SPR%s; SPR%s;SPRTMPLPW %s;SPRTMPLBTM %fDB; SPRJDG %s",
        	FreqSetArr[freqSetting], DetArr[detector],peakMkrYDelta, dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;	    	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Set                                    */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE],wrtBuf2[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    


    if (avopt63_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt63_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt63_invalidViReal64Range (deltaXHz, -100e+6, 100e+6))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<SPRTMPL %s; SPRTMPLSX %fHZ; SPRTMPLSY %fDB; SPRTMPLDX %fHZ",switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Edit                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLeveldB)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;   

    if (avopt63_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<SPRTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt63_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<SPRTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<SPRTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;   
}    


/*===========================================================================*/
/* Function: InBand Spurious Marker Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPowerdB)
{
 	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;    

	if (avopt63_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<SPRMKRCP");	
			break;
			
		case 1: /* Entry */
    			if (avopt63_invalidViReal64Range(startFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER5;
    			if (avopt63_invalidViReal64Range(stopFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    			if (avopt63_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER7;

            	Fmt (wrtBuf, "%s<SPRMKRED 0,%fHZ,%fHZ,%fDB", startFreqHz, stopFreqHz, limitPowerdB);
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<SPRMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt63_invalidViBooleanRange (refMarkerType))
        		return VI_ERROR_PARAMETER3;
    		if (avopt63_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;     
			Fmt (wrtBuf, "%s<SPRMKRCLR; SPRMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	
	total += NumFmtdBytes();
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Parameter Set                                  */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if (avopt63_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDSSETSTD");	
	}
	else
	{
	
    	if (avopt63_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt63_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER4;           	
   		if (avopt63_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER5;
   		if (avopt63_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6; 
   		if (avopt63_invalidViBooleanRange (preselector))
        	return VI_ERROR_PARAMETER7; 

        Fmt (wrtBuf, "%s<FDS%s;FDSPKMKY %fENT; FDS%s; FDSJDG %s; FDSPRE %s",
        	DetArr[detector], peakMkrYDelta,
        	LvlUnitArr[displayUnit], switchArr[judgement],preselArr[preselector]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Table Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel)
{
 	ViUInt32 total = 0;       
    ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	
	if (avopt63_invalidViInt16Range (operation, 0, 4))
        return VI_ERROR_PARAMETER2;
	if (avopt63_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

    switch (operation)
    {
    	case 0: /* Load */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSLD",tableNumber);
    		break;
    	case 1: /* Save */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSSV",tableNumber);
    		break;
    	case 2: /* Sel */
    		Fmt (wrtBuf, "%s<FDSTBL %d",tableNumber);
    		break;
    	case 3: /* Edit */
    		if (avopt63_invalidViReal64Range (startFreqHz, -200e+6, 8e+9))
        		return VI_ERROR_PARAMETER4;
			if (avopt63_invalidViReal64Range (stopFreqHz, 0.0, 8.2e+9))
        		return VI_ERROR_PARAMETER5;
			if (avopt63_invalidViReal64Range (RBW, 10.0, 10e+6))
        		return VI_ERROR_PARAMETER6;
        	if (avopt63_invalidViReal64Range (VBW, 1.0, 10e+6))
        		return VI_ERROR_PARAMETER7;
        	if (avopt63_invalidViReal64Range (sweepTime, 1e-6, 1000))
        		return VI_ERROR_PARAMETER8;        	
        	if (avopt63_invalidViReal64Range (limitLevel, -155, 65))
        		return VI_ERROR_INSTR_PARAMETER9;

     		Fmt (wrtBuf, "%s<FDSTBLED %fHZ,%fHZ, %fHZ, %fHZ, %fSC,%fDB",
     			startFreqHz,stopFreqHz,RBW,VBW,sweepTime,limitLevel);
    		break;
    	case 4: /* Init */
    		Fmt (wrtBuf, "%s<FDSCLR");
    	
    		break;
    }
   total += NumFmtdBytes();
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: Modulation Parameter Set GSM                                    */
/* Purpose:  This function sets the parameters for Modulation GSM			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setGSM (ViSession instrumentHandle,
                                           ViInt16 triggerSource,
                                           ViBoolean burstSwitch,
                                           ViReal64 searchLevel,
                                           ViBoolean trgSlope,
                                           ViInt16 triggerLevel,
                                           ViBoolean TDMAStructure,
                                           ViBoolean extTrgDelay,
                                           ViInt16 slotNumber,
                                           ViReal64 delayTimes,
                                           ViBoolean rootNyquistFilter)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ( avopt63_invalidViInt16Range(triggerSource,0,2))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViBooleanRange(burstSwitch))
        return VI_ERROR_PARAMETER3;
    if (avopt63_invalidViReal64Range(searchLevel, -40.0, -5.0))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViBooleanRange(trgSlope))
        return VI_ERROR_PARAMETER5;
    if (avopt63_invalidViInt16Range(triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER6;
    if (avopt63_invalidViBooleanRange(TDMAStructure))
        return VI_ERROR_PARAMETER7;
    if (avopt63_invalidViBooleanRange(extTrgDelay))
        return VI_ERROR_PARAMETER8;
    if (avopt63_invalidViInt16Range(slotNumber, 0, 7))
        return VI_ERROR_INSTR_PARAMETER9;
        
	if (avopt63_invalidViBooleanRange(rootNyquistFilter))
        return VI_ERROR_INSTR_PARAMETER11;

    Fmt (wrtBuf, "%s<MODTRG %s; MODTRGBRST %s; MODBRSTLVL %f; MODTRGSLP %s;MODTRGLVL %d; TRGSTR %s",
    	gsmTrgArr[triggerSource],
    	switchArr[burstSwitch],
    	searchLevel,
    	slopeArr[trgSlope],
    	triggerLevel,
    	tdmaStrArr[TDMAStructure]);
    
     total += NumFmtdBytes();   
    
    if(extTrgDelay==1)
    {
    	Fmt (wrtBuf, "%s[a]<; MODTRGSLT %d",
    		  slotNumber);
    }
    else
    {
    	Fmt (wrtBuf, "%s[a]<; MODTRGDLY %f",
    		 delayTimes);
    }
    
    total += NumFmtdBytes();   
    Fmt (wrtBuf, "%s[a]<; MODRNYQ %s",
    	switchArr[rootNyquistFilter]
    	);

    total += NumFmtdBytes();
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;     
}

/*===========================================================================*/
/* Function: Power vs Time Parameter Set	                                */
/* Purpose:  This function sets the parameters for Power vs Time Meas. 		*/
/*           																 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_powerTimeMeas_set (ViSession instrumentHandle,
                                            ViBoolean powervsTimeMeasMode,
                                            ViInt16 YScale)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	if (avopt63_invalidViBooleanRange(powervsTimeMeasMode))
        return VI_ERROR_PARAMETER2;
   if ( avopt63_invalidViInt16Range(YScale,0,2))
        return VI_ERROR_PARAMETER3;
    Fmt (wrtBuf, "%s<PTMOD %s; PTDIV %s",
    	pwrVsTimeMmArr[powervsTimeMeasMode], yScaleGSMArr[YScale]
    	);

    total += NumFmtdBytes();


	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;         
}

/*===========================================================================*/
/* Function: Power vs Time Template Edit GSM                                 */
/* Purpose:  This function sets the parameters for template GSM.			 */
/*           																 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_templatePwrTime_editGSM (ViSession instrumentHandle,
                                                  ViInt16 templateEntry,
                                                  ViReal64 y0Lvl,
                                                  ViBoolean y0Unit,
                                                  ViReal64 y1Lvl,
                                                  ViBoolean y1Unit,
                                                  ViReal64 y2Lvl,
                                                  ViBoolean y2Unit,
                                                  ViReal64 y3Lvl,
                                                  ViBoolean y3Unit,
                                                  ViReal64 y4Lvl,
                                                  ViBoolean y4Unit,
                                                  ViReal64 y5Lvl,
                                                  ViBoolean y5Unit,
                                                  ViReal64 y6Lvl,
                                                  ViBoolean y6Unit, 
                                                  ViReal64 t0,
                                                  ViReal64 t1, ViReal64 t2,
                                                  ViReal64 t3, ViReal64 t4)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if ( avopt63_invalidViInt16Range(templateEntry,0,3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViReal64Range (y0Lvl, -99.0, -0.1))
        return VI_ERROR_PARAMETER3;               
    if ( avopt63_invalidViBooleanRange(y0Unit))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViReal64Range (y1Lvl, -99.0, 20.0))
        return VI_ERROR_PARAMETER5;               
    if ( avopt63_invalidViBooleanRange(y1Unit))
        return VI_ERROR_PARAMETER6;
    if (avopt63_invalidViReal64Range (y2Lvl, -99.0, 20.0))
        return VI_ERROR_PARAMETER7;               
    if ( avopt63_invalidViBooleanRange(y2Unit))
        return VI_ERROR_PARAMETER8;
    if (avopt63_invalidViReal64Range (y3Lvl, -99.0, 20.0))  
        return VI_ERROR_INSTR_PARAMETER9;               
    if ( avopt63_invalidViBooleanRange(y3Unit))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt63_invalidViReal64Range (y4Lvl, -99.0, 20.0))  
        return VI_ERROR_INSTR_PARAMETER11;               
    if ( avopt63_invalidViBooleanRange(y4Unit))
        return VI_ERROR_INSTR_PARAMETER12;
    if (avopt63_invalidViReal64Range (y5Lvl, -99.0, 20.0))  
        return VI_ERROR_INSTR_PARAMETER13;               
    if ( avopt63_invalidViBooleanRange(y5Unit))
        return VI_ERROR_INSTR_PARAMETER14;
	if (avopt63_invalidViReal64Range (y6Lvl, -99.0, 20.0))  
        return VI_ERROR_INSTR_PARAMETER14;               
    if ( avopt63_invalidViBooleanRange(y6Unit))
        return VI_ERROR_INSTR_PARAMETER15;
    if (avopt63_invalidViReal64Range (t0, 0, 1e-5))
        return VI_ERROR_INSTR_PARAMETER16;                  
    if (avopt63_invalidViReal64Range (t1, 0, 18.0e-6))
        return VI_ERROR_INSTR_PARAMETER17;                  
    if (avopt63_invalidViReal64Range (t2, 0, 1e-5))
        return VI_ERROR_INSTR_PARAMETER18;                  
    if (avopt63_invalidViReal64Range (t3, 0, 9.0e-6))
        return VI_ERROR_INSTR_PARAMETER19;
    if (avopt63_invalidViReal64Range (t4, 0, 9.0e-6))
        return VI_ERROR_INSTR_PARAMETER20;


	Fmt (wrtBuf, "%s<GPTTYP %d", templateEntry+1);
	total += NumFmtdBytes();     
 	if (templateEntry  != 3)
 	{
 	
 	  Fmt (wrtBuf, "%s[a]<; GPTLVENT %fDB,%s,%fDB,%s,%fDB,%s,%fDB,%s,%fDB,%s,%fDB,%s,%fDB,%s; GPTTMENT %fSC,%fSC,%fSC,%fSC,%fSC",
 	  	   y0Lvl,
 	  	   zeroOneArr[y0Unit],
 	  	   y1Lvl,
 	  	   zeroOneArr[y1Unit],
		   y2Lvl,
 	  	   zeroOneArr[y2Unit],
		   y3Lvl,
 	  	   zeroOneArr[y3Unit],
		   y4Lvl,
 	  	   zeroOneArr[y4Unit],
		   y5Lvl,
 	  	   zeroOneArr[y5Unit],
 	  	   y6Lvl,
 	  	   zeroOneArr[y6Unit],
 	  	   t0,t1,t2,t3,t4);
 	  total += NumFmtdBytes();     	   
 	}

	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;         
}


/*===========================================================================*/
/* Function: Modulation Parameter Set DECT                                   */
/* Purpose:  This function sets the parameters for Modulation DECT			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_parameter_setDECT (ViSession instrumentHandle,
                                            ViInt16 triggerSource,
                                            ViBoolean burstSwitch,
                                            ViBoolean trgSlope,
                                            ViInt16 triggerLevel,
                                            ViBoolean extTrgDelay,
                                            ViInt16 fullSlotNumber,
                                            ViInt16 halfSlotNumber,
                                            ViReal64 delayTimes,
                                            ViBoolean freqDeviationBITSequence,
                                            ViBoolean timingJitterRfptoPP)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if ( avopt63_invalidViInt16Range(triggerSource,0,2))
        return VI_ERROR_PARAMETER2;
    if ( avopt63_invalidViBooleanRange(burstSwitch))
        return VI_ERROR_PARAMETER3;
    if ( avopt63_invalidViBooleanRange(trgSlope))
        return VI_ERROR_PARAMETER4;    
    if ( avopt63_invalidViInt16Range(triggerLevel,0,100))
        return VI_ERROR_PARAMETER5;
    if ( avopt63_invalidViBooleanRange(extTrgDelay))
        return VI_ERROR_PARAMETER6; 
    if ( avopt63_invalidViInt16Range(fullSlotNumber,0,23))
        return VI_ERROR_PARAMETER7;        
    if ( avopt63_invalidViInt16Range(halfSlotNumber,0,1))
        return VI_ERROR_PARAMETER8;        
        
    if ( avopt63_invalidViBooleanRange(freqDeviationBITSequence))
        return VI_ERROR_INSTR_PARAMETER10;     
    if ( avopt63_invalidViBooleanRange(timingJitterRfptoPP))
        return VI_ERROR_INSTR_PARAMETER11;     
    
    
    Fmt (wrtBuf, "%s<MODTRG %s; MODTRGBRST %s; MODTRGSLP %s; MODTRGLVL %d",
    	gsmTrgArr[triggerSource],
    	switchArr[burstSwitch],
    	slopeArr[trgSlope],
    	triggerLevel);

    total += NumFmtdBytes();

    if(extTrgDelay==1)
    {
    	Fmt (wrtBuf, "%s[a]<; FULSLT %d; HAFSLT %d",
    		 fullSlotNumber,
    		 halfSlotNumber);
    }
    else
    {
    	Fmt (wrtBuf, "%s[a]<; MODTRGDLY %f",
    		 delayTimes);
    }
    
    total += NumFmtdBytes();  	

	Fmt (wrtBuf, "%s[a]<; DFMBITSEQ %s; DTJLNK %s",
    	dfmBitSeqArr[freqDeviationBITSequence],
    	switchArr[timingJitterRfptoPP]
    	);
	
	total += NumFmtdBytes();  
	
	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;       
}

/*===========================================================================*/
/* Function: Power vs Time Template Edit DECT                                */
/* Purpose:  This function sets the parameters for template DECT.			 */
/*           																 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_templatePwrTime_editDECT (ViSession instrumentHandle,
                                                   ViInt16 templateEntry,
                                                   ViReal64 y0Lvl,
                                                   ViBoolean y0Unit,
                                                   ViReal64 y1Lvl,
                                                   ViBoolean y1Unit,
                                                   ViReal64 y2Lvl,
                                                   ViBoolean y2Unit,
                                                   ViReal64 y3Lvl,
                                                   ViBoolean y3Unit,
                                                   ViReal64 y4Lvl,
                                                   ViBoolean y4Unit,
                                                   ViReal64 y5Lvl,
                                                   ViBoolean y5Unit)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if ( avopt63_invalidViInt16Range(templateEntry,0,3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViReal64Range (y0Lvl, -99.0, -0.1))
        return VI_ERROR_PARAMETER3;               
    if ( avopt63_invalidViBooleanRange(y0Unit))
        return VI_ERROR_PARAMETER4;
    if (avopt63_invalidViReal64Range (y1Lvl, -99.0, 20.0))
        return VI_ERROR_PARAMETER5;               
    if ( avopt63_invalidViBooleanRange(y1Unit))
        return VI_ERROR_PARAMETER6;
    if (avopt63_invalidViReal64Range (y2Lvl, -99.0, 20.0))
        return VI_ERROR_PARAMETER7;               
    if ( avopt63_invalidViBooleanRange(y2Unit))
        return VI_ERROR_PARAMETER8;
    if (avopt63_invalidViReal64Range (y3Lvl, -99.0, 20.0))  
        return VI_ERROR_INSTR_PARAMETER9;               
    if ( avopt63_invalidViBooleanRange(y3Unit))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt63_invalidViReal64Range (y4Lvl, -99.0, 20.0))  
        return VI_ERROR_INSTR_PARAMETER11;               
    if ( avopt63_invalidViBooleanRange(y4Unit))
        return VI_ERROR_INSTR_PARAMETER12;
    if (avopt63_invalidViReal64Range (y5Lvl, -99.0, 20.0))  
        return VI_ERROR_INSTR_PARAMETER13;               
    if ( avopt63_invalidViBooleanRange(y5Unit))
        return VI_ERROR_INSTR_PARAMETER14;
                 

	Fmt (wrtBuf, "%s<DPTTYP %d", templateEntry+1);
	total += NumFmtdBytes();
	
 	if (templateEntry  != 3)
 	{
 	
 	  Fmt (wrtBuf, "%s[a]<; DPTLVENT %fDB,%s,%fDB,%s,%fDB,%s,%fDB,%s,%fDB,%s,%fDB,%s",
 	  	   y0Lvl,
 	  	   zeroOneArr[y0Unit],
 	  	   y1Lvl,
 	  	   zeroOneArr[y1Unit],
		   y2Lvl,
 	  	   zeroOneArr[y2Unit],
		   y3Lvl,
 	  	   zeroOneArr[y3Unit],
		   y4Lvl,
 	  	   zeroOneArr[y4Unit],
		   y5Lvl,
 	  	   zeroOneArr[y5Unit]);
 	  total += NumFmtdBytes();     	   
 	}

	if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;             
}

/*===========================================================================*/
/* Function: Averaging                                                       */
/* Purpose:  This function sets the averaging times of Burst Envelope		 */
/*           Spectrum, Gated Output Power, On/Off Ratio, OBW,  Channel Power,*/
/*           In Band Spurious, Waveform Quality, Code Domain Power, TX Power */
/*           functions.														 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_average_setOpt63 (ViSession instrumentHandle,
                                           ViInt16 function, ViInt16 AVGTimes)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (function, 0, 8))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViInt16Range (AVGTimes, 1, 999))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s %d", avgFunctionArr[function], AVGTimes);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: GSM Averaging                                                   */
/* Purpose:  This function sets the averaging times of DECT Modulation menu. */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_averaging_setGSM (ViSession instrumentHandle,
                                           ViInt16 function, ViInt16 AVGTimes)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (function, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViInt16Range (AVGTimes, 1, 999))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s %d", avgFuncGsmArr[function], AVGTimes);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;        
}

/*===========================================================================*/
/* Function: DECT Averaging                                                  */
/* Purpose:  This function sets the averaging times of DECT Modulation menu. */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_averaging_setDECT (ViSession instrumentHandle,
                                             ViInt16 function, ViInt16 AVGTimes)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (function, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt63_invalidViInt16Range (AVGTimes, 1, 999))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s %d", avgFuncDectArr[function], AVGTimes);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: Auto Level Set                                                  */
/* Purpose:  This function is used for Auto Level set in selected meas mode. */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (function, 0, 9))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", autoLvlFunctionArr[function]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}


/*===========================================================================*/
/* Function: DC Cal					                                         */
/* Purpose:  This function execute DC Cal.									 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_act_DcCal (ViSession instrumentHandle)
{
    ViUInt32 retCnt = 0;
    ViStatus avopt63_status = VI_SUCCESS;
    
    if ((avopt63_status = viWrite (instrumentHandle, "CLDC", 4, &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: Opt63 Start Measurement                                         */
/* Purpose:  This function starts desired measurement mode of TRANSIENT		 */
/*           section.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_measStart_actOpt63 (ViSession instrumentHandle,
                                             ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (function, 0, 9))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", funcMeasureArr[function]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}


/*===========================================================================*/
/* Function: GSM Start Measurement                                           */
/* Purpose:  This function starts desired measurement mode of TRANSIENT		 */
/*           section.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_measStart_actGSM (ViSession instrumentHandle,
                                           ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (function, 0, 4))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", funcMeasGsmArr[function]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;    
}


/*===========================================================================*/
/* Function: GSM Start Measurement                                           */
/* Purpose:  This function starts desired measurement mode of TRANSIENT		 */
/*           section.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_measStart_actDECT (ViSession instrumentHandle,
                                            ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (function, 0, 4))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", funcMeasDectArr[function]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;        
}

/*===========================================================================*/
/* Function: Graph Type Selection (Phase Error).                             */
/* Purpose:  This function is used for graph type selection.				 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_graphType_actPErr (ViSession instrumentHandle,
                                            ViInt16 graphType)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (graphType, 0, 12))
        return VI_ERROR_PARAMETER2;
 
        
    Fmt (wrtBuf, "%s<%s",gTypPEgsmArr[graphType]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;        
}

/*===========================================================================*/
/* Function: Graph Type Selection  (GSM)                                     */
/* Purpose:  This function is used for graph type selection.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_graphType_actGSM (ViSession instrumentHandle,
                                           ViInt16 graphType)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (graphType, 0, 17))
        return VI_ERROR_PARAMETER2;
 
        
    Fmt (wrtBuf, "%s<%s",gTypGsmArr[graphType]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;     
}


/*===========================================================================*/
/* Function: Graph Type Selection  (DECT)                                    */
/* Purpose:  This function is used for graph type selection.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_graphType_actDECT (ViSession instrumentHandle,
                                            ViInt16 graphType, ViInt16 startBit)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViInt16Range (graphType, 0, 2))
        return VI_ERROR_PARAMETER2;
        
    if (avopt63_invalidViInt16Range (startBit, -50, 18))
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuf, "%s<DSTTBIT %d; %s",startBit,gTypDectArr[graphType]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;     
}

/*===========================================================================*/
/* Function: Graph Marker                                                    */
/* Purpose:  This function is used to switch graph marker On/Off.			 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_graphMkr_act (ViSession instrumentHandle,
                                       ViBoolean marker)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if (avopt63_invalidViBooleanRange (marker))
        return VI_ERROR_PARAMETER2;

	if (marker)
	{
	 Fmt (wrtBuf, "%s<MK");   	
	}
	else
	{
	 Fmt (wrtBuf, "%s<MKOFF");
	}

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt63_status;

    return avopt63_status;
}


/*===========================================================================*/
/* Function: Check meas mode		                                         */
/* Purpose:  This function returns the mode of instrument.(CW/Transient)     */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    

    if ((avopt63_status = viWrite (instrumentHandle, "SETFUNC?", 8, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", returnmode) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;	       
    
}

/*===========================================================================*/
/* Function: Check Mode GSM													 */
/* Purpose:	This function returns the Communication Type.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_check_modeGSM (ViSession instrumentHandle,
                                        ViInt16 *link, ViInt16 *system,
                                        ViInt16 *modulation)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;
    
    if ((avopt63_status = viWrite (instrumentHandle, "MODTYP?", 7, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", system) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt63_status = viWrite (instrumentHandle, "LINK?", 5, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;
   	if (Scan (rdBuf, "%s>%d[b2]", link) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avopt63_status = viWrite (instrumentHandle, "MODSYS?", 7, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", modulation) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;	            
}

/*===========================================================================*/
/* Function: Time Domain Power Result                                        */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "TDPMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;	    
}

/*===========================================================================*/
/* Function: On/Off Ratio                                                    */
/* Purpose:  This function returns the result of On/Off ratio measurement of */
/*           the burst signal in the time domain.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_OnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito,
                                          ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "OORMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%d[b2]", ONLevel, OFFLevel, ONOFFRaito, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;    
}


/*===========================================================================*/
/* Function: TD Spurious Data                                                */
/* Purpose:  This function returns the result of Time Domain Spurious		 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], ViReal64 level[],
                                       ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "TDSMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", tableSize) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*tableSize;i++) {
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt63_status;
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &level[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt63_status;
}

/*===========================================================================*/
/* Function: F Domain Power Result                                        */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "FDPMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;	        
}

/*===========================================================================*/
/* Function: OBW Data                                                        */
/* Purpose:  This function returns the result of occupied bandwidth			 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_data_OBW (ViSession instrumentHandle, ViReal64 *OBWHz,
                                   ViReal64 *lowFreqHz, ViReal64 *hiFreqHz,
                                   ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "OBWMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%d[b2]", OBWHz, lowFreqHz, hiFreqHz, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;    
}

/*===========================================================================*/
/* Function: Due to Transient Data                                           */
/* Purpose:  This function returns the result of Due to Transient			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "DTSMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;

    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt63_status;
	   	if (Scan (rdBuf, "%s>%f,%d[b2]", &ACPDatadBc[i], &judgement[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
	
    return avopt63_status;    
}

/*===========================================================================*/
/* Function: Due to Modulation Data                                          */
/* Purpose:  This function returns the result of Due to Modulation			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "DTMMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt63_status;
	   	if (Scan (rdBuf, "%s>%f,%d[b2]", &ACPdBc[i], &ACPJudge[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
    return avopt63_status;    	    
}

/*===========================================================================*/
/* Function: In Band Spurious                                                */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_INSPR (ViSession instrumentHandle,
                                     ViInt16 *numberofPoint,
                                     ViReal64 frequencyHz[], ViReal64 leveldBm[],
                                     ViInt16 judge[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "SPRMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
        
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt63_status;
	   	
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &leveldBm[i], &judge[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt63_status;
}

/*===========================================================================*/
/* Function: Out Band Spurious                                               */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "FDSMEAS?", 8, &retCnt)) < 0)
        return avopt63_status;
        
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt63_status;
	   	
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &leveldBm[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt63_status;		
}



/*===========================================================================*/
/* Function: TX Power DECT                                                   */
/* Purpose:  This function returns the result of TX power measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_TxPowerDECT (ViSession instrumentHandle,
                                       ViReal64 *powerLeveldBm,
                                       ViReal64 *powerLevelW)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "TXPWR?", 6, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f", powerLeveldBm, powerLevelW) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;    
}


/*===========================================================================*/
/* Function: TX Power GSM                                                    */
/* Purpose:  This function returns the result of TX power measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_TxPowerGSM (ViSession instrumentHandle,
                                          ViInt16 *modulation,
                                          ViReal64 *powerLeveldBm,
                                          ViReal64 *powerLevelW,
                                          ViReal64 *peakFactordB)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "MODSYS?", 7, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;
	if (Scan (rdBuf, "%s>%d[b2]", modulation) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	
   	if ((avopt63_status = viWrite (instrumentHandle, "TXPWR?", 7, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;
   	
   	if (*modulation == 1)
   	{
   		if (Scan (rdBuf, "%s>%f,%f,%f", powerLeveldBm, powerLevelW,peakFactordB) < 3)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
   	else
   	{
   		if (Scan (rdBuf, "%s>%f,%f", powerLeveldBm, powerLevelW) < 2)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt63_status;		   	
}

/*===========================================================================*/
/* Function: Phase Error Data				 								 */
/* Purpose:  This function returns the result of GSM phase error 			 */
/*			 measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_dataPhaseError (ViSession instrumentHandle,
                                          ViReal64 *phaseError,
                                          ViReal64 *peakPhaseError,
                                          ViReal64 *peakPhaseErrorPosition,
                                          ViReal64 *frequencyError)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "PHACC2?", 7, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%f", phaseError, peakPhaseError, peakPhaseErrorPosition, frequencyError) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;
}

/*===========================================================================*/
/* Function: Phase Error Graphic Data										 */
/* Purpose:  This function returns the graphic data of GSM phase error		 */
/*			 measurement.													 */
/*===========================================================================*/

static ViString PhsErrGraphArr[] = {"INP","LIN","DOT","CON","IEYE","QEYE",
									"IQEYE","TR","PE","FPE","BF","FREYE"};

ViStatus CVIFUNC avopt63_dataGraphPhaseError (ViSession instrumentHandle,
                                               ViInt16 graphType,
                                               ViInt16 *xDataSize, ViReal64 xData[],
                                               ViInt16 *yDataSize, ViReal64 yData[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE],wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Graph Type Select */
	if (avopt63_invalidViInt16Range (graphType, 0, 11))
		return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<GGPTYP %s", PhsErrGraphArr[graphType]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt63_status;
        
    Delay (1.0);    
        
	/* X Data */
	if ( graphType < 7)
	{
    	if ((avopt63_status = viWrite (instrumentHandle, "GPHI?", 5, &retCnt)) < 0)
        	return avopt63_status;
	}
	else
	{
		if ((avopt63_status = viWrite (instrumentHandle, "GPHX?", 5, &retCnt)) < 0)
        	return avopt63_status;
	}
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", xDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *xDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%f", &xData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
	/* Y Data */
	if ( graphType < 7)
	{
    	if ((avopt63_status = viWrite (instrumentHandle, "GPHQ?", 5, &retCnt)) < 0)
        	return avopt63_status;
	}
	else
	{
		if ((avopt63_status = viWrite (instrumentHandle, "GPHY?", 5, &retCnt)) < 0)
        	return avopt63_status;
	}
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", yDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *yDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%f", &yData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
	return avopt63_status;
}

/*===========================================================================*/
/* Function: Phase Error Demodulated Data									 */
/* Purpose:  This function returns the demodulated data of GSM phase error	 */
/*			 measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_dataDmlPhaseError (ViSession instrumentHandle,
                                            ViInt16 *demodulatedDataSize,
                                            ViInt32 demodulatedData[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Display Demodulated Data */
	if ((avopt63_status = viWrite (instrumentHandle, "GGPTYP DE", 9, &retCnt)) < 0)
		return avopt63_status;
	
	Delay (1.0);
	
	/* Read Demodulated Data */
   	if ((avopt63_status = viWrite (instrumentHandle, "DEMOD?", 6, &retCnt)) < 0)
       	return avopt63_status;
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", demodulatedDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *demodulatedDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%d", &demodulatedData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
	return avopt63_status;
}


/*===========================================================================*/
/* Function: Modulation Accuracy Data										 */
/* Purpose:  This function returns the results of GSM modulation accuracy	 */
/*			 measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_dataModAcc (ViSession instrumentHandle,
                                      ViReal64 *burstAmpDroop,
                                      ViReal64 *crrFreqErr,
                                      ViReal64 *IQOriginOffset,
                                      ViReal64 *magnitudeErr,
                                      ViReal64 *phaseErr,
                                      ViReal64 *errorVectorMag,
                                      ViReal64 *per_95th,
                                      ViReal64 *avgdPeakEVM,
                                      ViReal64 *peakMagErr,
                                      ViInt16 *pos_PeakMagErr,
                                      ViReal64 *peakPhsErr,
                                      ViInt16 *pos_PeakPhsErr,
                                      ViReal64 *peakEVM,
                                      ViInt16 *pos_PeakEVM)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "MODACC?", 7, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f,%f,%f", burstAmpDroop, crrFreqErr,
   			IQOriginOffset, magnitudeErr, phaseErr, errorVectorMag, per_95th, avgdPeakEVM) < 8)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	
    if ((avopt63_status = viWrite (instrumentHandle, "MODACCPK?", 9, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%d[b2],%f,%d[b2],%f,%d[b2]", peakMagErr, pos_PeakMagErr,
   			peakPhsErr, pos_PeakPhsErr, peakEVM, pos_PeakEVM) < 6)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	return avopt63_status;    
}

/*===========================================================================*/
/* Function: Modulation Accuracy Graphic Data								 */
/* Purpose:  This function returns the graphic data of GSM modulation		 */
/*			 accuracy measurement.											 */
/*===========================================================================*/

static ViString ModAccGraphArr[] = {"INP","LIN","DOT","CON","IEYE","QEYE",
									"IQEYE","EVM","ME","PFE","8INP","8LIN",
									"8DOT","8CON","8IEYE","8QEYE","8IQEYE"};

ViStatus CVIFUNC avopt63_dataGraphModAcc (ViSession instrumentHandle,
                                           ViInt16 graphType, 
                                           ViInt16 *xDataSize, ViReal64 xData[], 
                                           ViInt16 *yDataSize, ViReal64 yData[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE],wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Graph Type Select */
	if (avopt63_invalidViInt16Range (graphType, 0, 16))
		return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<EGPTYP %s", ModAccGraphArr[graphType]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt63_status;
        
    Delay (1.0);
    
	/* X Data */
	if ( graphType>=7 && graphType<=9)
	{
    	if ((avopt63_status = viWrite (instrumentHandle, "EGPHX?", 6, &retCnt)) < 0)
        	return avopt63_status;
	}
	else
	{
		if ((avopt63_status = viWrite (instrumentHandle, "EGPHI?", 6, &retCnt)) < 0)
        	return avopt63_status;
	}
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", xDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *xDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%f", &xData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
    
	/* Y Data */
	if ( graphType>=7 && graphType<=9)
	{
    	if ((avopt63_status = viWrite (instrumentHandle, "EGPHY?", 6, &retCnt)) < 0)
        	return avopt63_status;
	}
	else
	{
		if ((avopt63_status = viWrite (instrumentHandle, "EGPHQ?", 6, &retCnt)) < 0)
        	return avopt63_status;
	}
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", yDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *yDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%f", &yData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
	return avopt63_status;
}

/*===========================================================================*/
/* Function: Modulation Accuracy Demodulated Data							 */
/* Purpose:  This function returns the demodulated data of GSM modulation	 */
/*			 accuracy measurement.											 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_dataDmlModAcc (ViSession instrumentHandle,
                                        ViInt16 *demodulatedDataSize,
                                        ViInt32 demodulatedData[])
{
   ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Display Demodulated Data */
	if ((avopt63_status = viWrite (instrumentHandle, "EGPTYP DEMOD", 12, &retCnt)) < 0)
		return avopt63_status;
	
	Delay (1.0);
	
	/* Read Demodulated Data */
   	if ((avopt63_status = viWrite (instrumentHandle, "EDEMOD?", 7, &retCnt)) < 0)
       	return avopt63_status;
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", demodulatedDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *demodulatedDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%d", &demodulatedData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
	return avopt63_status;
}


/*===========================================================================*/
/* Function: Freq Deviation Random Sequence Data							 */
/* Purpose:  This function returns the result of the Frequency Deviation	 */
/*			 measurement. Bit Sequence is set to RANDOM.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_dataFreqDevRandSeq (ViSession instrumentHandle,
                                              ViReal64 *freqErr,
                                              ViReal64 *maxPeakDev,
                                              ViReal64 *minPeakDev)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "DFMDEV?", 7, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f", freqErr, maxPeakDev, minPeakDev) < 3)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;
}

/*===========================================================================*/
/* Function: Freq Deviation STD Sequence Data								 */
/* Purpose:  This function returns the result of the Frequency Deviation	 */
/*			 measurement. Bit Sequence is set to STD.						 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_dataFreqDevSTDSeq (ViSession instrumentHandle,
                                            ViReal64 *frequencyErrorHz,
                                            ViReal64 *syncFieldMaxPeakDevHz,
                                            ViReal64 *syncFieldMinPeakDevHz,
                                            ViReal64 *lpbackFieldMaxPeakDevHz,
                                            ViReal64 *lpbackFieldMinPeakDevHz)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viWrite (instrumentHandle, "DFMDEVSTD?", 10, &retCnt)) < 0)
        return avopt63_status;
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f", frequencyErrorHz, 
   			syncFieldMaxPeakDevHz, syncFieldMinPeakDevHz, 
			lpbackFieldMaxPeakDevHz, lpbackFieldMinPeakDevHz) < 5)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt63_status;
}

/*===========================================================================*/
/* Function: Freq Deviation Graphic Data									 */
/* Purpose:  This function returns the graphic data of DECT frequency		 */
/*			 deviation measurement.											 */
/*===========================================================================*/

static ViString FreqDevGraphArr[] = {"BIT","EYE"};


ViStatus CVIFUNC avopt63_dataGraphFreqDeviation (ViSession instrumentHandle,
                                                  ViInt16 graphType,
                                                  ViInt16 *XDataSize, ViReal64 XData[],
                                                  ViInt16 *YDataSize, ViReal64 YData[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE],wrtBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Graph Type Select */
	if (avopt63_invalidViInt16Range (graphType, 0, 1))
		return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<DFMGTYP %s", FreqDevGraphArr[graphType]);

    if ((avopt63_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt63_status;
        
    Delay (1.0);
    
	/* X Data */
   	if ((avopt63_status = viWrite (instrumentHandle, "GPHX?", 5, &retCnt)) < 0)
       	return avopt63_status;
	
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", XDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *XDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%f", &XData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
    
	/* Y Data */
   	if ((avopt63_status = viWrite (instrumentHandle, "GPHY?", 5, &retCnt)) < 0)
       	return avopt63_status;
	
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", YDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *YDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%f", &YData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
    
    return avopt63_status;
}

/*===========================================================================*/
/* Function: Modulation Accuracy Demodulated Data							 */
/* Purpose:  This function returns the demodulated data of DECT frequency	 */
/*			 deviation measurement.											 */
/*===========================================================================*/

ViStatus CVIFUNC avopt63_dataDmlFreqDeviation (ViSession instrumentHandle,
                                               ViInt16 *demodulatedDataSize,
                                               ViInt32 demodulatedData[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Display Demodulated Data */
	if ((avopt63_status = viWrite (instrumentHandle, "DFMGTYP DEMOD", 13, &retCnt)) < 0)
		return avopt63_status;
	
	Delay (1.0);
	
	/* Read Demodulated Data */
   	if ((avopt63_status = viWrite (instrumentHandle, "DEMOD?", 6, &retCnt)) < 0)
       	return avopt63_status;
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", demodulatedDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *demodulatedDataSize; i++)
	{
	    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt63_status;
   	    	
   		if (Scan (rdBuf, "%s>%d", &demodulatedData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
	return avopt63_status;
}

/*===========================================================================*/
/* Function: DECT Timing Jitter Data										 */
/* Purpose:  This function returns the demodulated data of DECT frequency	 */
/*			 deviation measurement.											 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_TimJitDECT (ViSession instrumentHandle,
                                          ViReal64 *maxPeakJitter,
                                          ViReal64 *minPeakJitter,
                                          ViReal64 *averageJitter)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Read Demodulated Data */
   	if ((avopt63_status = viWrite (instrumentHandle, "DTJIT?", 6, &retCnt)) < 0)
       	return avopt63_status;
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f",maxPeakJitter, minPeakJitter, averageJitter) < 3 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	
	return avopt63_status;    
}


/*===========================================================================*/
/* Function: Power vs Time Data												 */
/* Purpose:  This function returns the result of  Power vs Time 			 */
/*			 measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_data_PwrVsTime (ViSession instrumentHandle,
                                         ViReal64 *leveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt63_status = VI_SUCCESS;

	/* Data? */
   	if ((avopt63_status = viWrite (instrumentHandle, "PWRTM?", 6, &retCnt)) < 0)
       	return avopt63_status;
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%f",leveldBm) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	/* Judge? */
	if ((avopt63_status = viWrite (instrumentHandle, "PTJDG?", 6, &retCnt)) < 0)
       	return avopt63_status;
		
    if ((avopt63_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt63_status;

   	if (Scan (rdBuf, "%s>%d[b2]", judgement) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	return avopt63_status;        
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC avopt63_close (ViSession instrSession)
{
    avopt63_instrRange instrPtr;
    ViSession rmSession;
    ViStatus avopt63_status = VI_SUCCESS;

    if ((avopt63_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avopt63_status;
    if ((avopt63_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return avopt63_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    avopt63_status = viClose (instrSession);
    viClose (rmSession);

    return avopt63_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avopt63_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt63_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt63_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt63_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt63_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt63_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt63_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt63_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*================================================================================*/
/* Function: Wait On VISA Event                                                   */
/* Purpose:  This function waits for the occurrence of an enabled VISA Event. The */
/*           event type must be enabled before entering this function. Any        */
/*           programmatic commands, actions, or conditions, necessary to generate */
/*           the specified event type, must be satisified before entering this    */
/*           function. The function will wait for the specified timeout and then  */
/*           return. If the specified event is received then the function will    */
/*           return VI_SUCCESS (0), otherwise the status code for the generated   */
/*           error will be returned. In either case the function will disable the */
/*           event type and deallocate the event handle passed from viWaitOnEvent */
/*           before returning. WARNING: If VI_TMO_INFINITE is passed in as the    */
/*           timeout this function WILL NOT return until the specified event is   */
/*           received, if the event is not received the function will not return  */
/*           and it will be necessary to terminate CVI in order to regain control.*/
/*================================================================================*/
ViStatus avopt63_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, 
            ViUInt32 timeoutIn, ViPUInt16 STB)
{            
    ViStatus avopt63_status = VI_SUCCESS, tempStatus = VI_SUCCESS;
    ViEventType eventTypeOut = 0;
    ViEvent eventHandle = 0;
    
    /* For debug purposes we want to be able to see the status returned by        */
    /* viDisableEvent and viClose if one of the internal functions fails but do   */
    /* not want to return that value as that is not where the initial error       */
    /* occurs in the function, so we assign it to tempStatus.                     */
    
    if ((avopt63_status = viWaitOnEvent (instrSession, eventTypeIn, timeoutIn, 
                                        &eventTypeOut, &eventHandle)) < 0) {
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt63_status;
    } 
    
    if (eventTypeIn == VI_EVENT_SERVICE_REQ)
        if ((avopt63_status = viReadSTB (instrSession, STB)) < 0) {
            tempStatus = viClose (eventHandle);
            tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
            return avopt63_status;
        }   
        
    if ((avopt63_status = viClose (eventHandle)) < 0) {             
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt63_status;                 
    }
    
    if ((avopt63_status = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE)) < 0)
        return avopt63_status;                                                   

    return avopt63_status;                                    
}    

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avopt63_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avopt63_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus avopt63_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    avopt63_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the avopt63_reset operation and by the     */
/*           avopt63_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus avopt63_defaultInstrSetup (ViSession instrSession)
{
    ViStatus avopt63_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    avopt63_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (avopt63_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avopt63_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct avopt63_statusDataRanges));

    instrPtr -> traceRange = 1001;
    instrPtr -> harmonics = 3;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", avopt63_REVISION);
    
    if ((avopt63_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avopt63_status;                                       

    return avopt63_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
