#include <cvidef.h>

/*= R3267 Series - Modulation Spectrum Analyzes Include File ================*/

/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/
     
#ifndef __avopt63_HEADER
#define __avopt63_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER9               (_VI_ERROR+0x3FFC0809L)
#define VI_ERROR_INSTR_PARAMETER10              (_VI_ERROR+0x3FFC080AL)
#define VI_ERROR_INSTR_PARAMETER11              (_VI_ERROR+0x3FFC080BL)
#define VI_ERROR_INSTR_PARAMETER12              (_VI_ERROR+0x3FFC080CL)
#define VI_ERROR_INSTR_PARAMETER13              (_VI_ERROR+0x3FFC080DL)
#define VI_ERROR_INSTR_PARAMETER14              (_VI_ERROR+0x3FFC080EL)
#define VI_ERROR_INSTR_PARAMETER15              (_VI_ERROR+0x3FFC080FL)
#define VI_ERROR_INSTR_PARAMETER16              (_VI_ERROR+0x3FFC0810L)
#define VI_ERROR_INSTR_PARAMETER17              (_VI_ERROR+0x3FFC0811L)
#define VI_ERROR_INSTR_PARAMETER18              (_VI_ERROR+0x3FFC0812L)        
#define VI_ERROR_INSTR_PARAMETER19              (_VI_ERROR+0x3FFC0813L)
#define VI_ERROR_INSTR_PARAMETER20              (_VI_ERROR+0x3FFC0814L)

/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                           (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                   (_VI_ERROR+0x3FFC0900L)

#define avopt63_ERROR_INVALID_CONFIGURATION      (VI_ERROR_INSTR_OFFSET + 0xF0L)

/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/

ViStatus CVIFUNC avopt63_parameter_std_setGSM (ViSession instrumentHandle,
                                               ViBoolean modulation,
                                               ViInt16 type, ViInt16 measMode,
                                               ViBoolean link,
                                               ViBoolean burstType,
                                               ViBoolean syncType, ViInt16 TSC,
                                               ViBoolean fillterMode,
                                               ViInt16 powerCtrLvl,
                                               ViReal64 offsetLevel,
                                               ViBoolean frequencyInput,
                                               ViBoolean input,
                                               ViBoolean basebandInput,
                                               ViBoolean IQInverse,
                                               ViBoolean contAutoLevelSet);

ViStatus CVIFUNC avopt63_parameter_std_setDECT (ViSession instrumentHandle,
                                                ViBoolean measMode,
                                                ViBoolean link,
                                                ViInt16 physicalPacket,
                                                ViBoolean ZFiled,
                                                ViBoolean syncType,
                                                ViBoolean fillterMode,
                                                ViBoolean powerLevel,
                                                ViReal64 offsetLevel,
                                                ViBoolean frequencyInput,
                                                ViBoolean input,
                                                ViBoolean basebandInput,
                                                ViBoolean IQInverse,
                                                ViBoolean contAutoLevelSet);

ViStatus CVIFUNC avopt63_number_setChannel (ViSession instrumentHandle,
                                            ViInt16 channelNo);

ViStatus CVIFUNC avopt63_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViInt16 editChannel, ViInt16 startCHNum,
                                ViInt16 stopCHNum, ViReal64 baseFreqHz,
                                ViReal64 channelSpaceHz, ViInt16 CHOffset);

ViStatus CVIFUNC avopt63_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3);

ViStatus CVIFUNC avopt63_Yscale_setup (ViSession instrumentHandle,
                                       ViInt16 measMode, ViInt16 division);

ViStatus CVIFUNC avopt63_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes);

ViStatus CVIFUNC avopt63_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS);

ViStatus CVIFUNC avopt63_template_SetTDP (ViSession instrumentHandle,
                                          ViBoolean template, ViReal64 shiftXS,
                                          ViReal64 shiftYdB);

ViStatus CVIFUNC avopt63_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel);

ViStatus CVIFUNC avopt63_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt63_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt63_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS);

ViStatus CVIFUNC avopt63_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit);

ViStatus CVIFUNC avopt63_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt63_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level);

ViStatus CVIFUNC avopt63_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViBoolean result, ViReal64 peakMKRYDelta,
                                       ViReal64 multiplier,
                                       ViBoolean preselector);

ViStatus CVIFUNC avopt63_trigger_setGtSpc (ViSession instrumentHandle,
                                           ViInt16 trgSource, ViBoolean trgSlope,
                                           ViInt16 triggerLevel,
                                           ViInt16 triggerPosition,
                                           ViReal64 delayTimes,
                                           ViBoolean gateSource,
                                           ViBoolean gateSettoSTD,
                                           ViReal64 gatePositions,
                                           ViReal64 gateWidths,
                                           ViBoolean gatedSweep,
                                           ViInt16 detector);

ViStatus CVIFUNC avopt63_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz);

ViStatus CVIFUNC avopt63_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt63_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt63_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor);

ViStatus CVIFUNC avopt63_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt63_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt63_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt63_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor);

ViStatus CVIFUNC avopt63_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt63_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt63_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViBoolean refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt63_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement);

ViStatus CVIFUNC avopt63_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz);

ViStatus CVIFUNC avopt63_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt63_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt63_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector);

ViStatus CVIFUNC avopt63_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel);

ViStatus CVIFUNC avopt63_parameter_setGSM (ViSession instrumentHandle,
                                           ViInt16 triggerSource,
                                           ViBoolean burstSwitch,
                                           ViReal64 searchLevel,
                                           ViBoolean trgSlope,
                                           ViInt16 triggerLevel,
                                           ViBoolean TDMAStructure,
                                           ViBoolean extTrgDelay,
                                           ViInt16 slotNumber,
                                           ViReal64 delayTimes,
                                           ViBoolean rootNyquistFilter);

ViStatus CVIFUNC avopt63_powerTimeMeas_setGSM (ViSession instrumentHandle,
                                               ViBoolean powervsTimeMeasMode,
                                               ViInt16 YScale);

ViStatus CVIFUNC avopt63_templatePwrTime_editGSM (ViSession instrumentHandle,
                                                  ViInt16 templateEntry,
                                                  ViReal64 y0Lvl,
                                                  ViBoolean y0Unit,
                                                  ViReal64 y1Lvl,
                                                  ViBoolean y1Unit,
                                                  ViReal64 y2Lvl,
                                                  ViBoolean y2Unit,
                                                  ViReal64 y3Lvl,
                                                  ViBoolean y3Unit,
                                                  ViReal64 y4Lvl,
                                                  ViBoolean y4Unit,
                                                  ViReal64 y5Lvl,
                                                  ViBoolean y5Unit,
                                                  ViReal64 y6Lvl,
                                                  ViBoolean y6Unit, ViReal64 t0,
                                                  ViReal64 t1, ViReal64 t2,
                                                  ViReal64 t3, ViReal64 t4);

ViStatus CVIFUNC avopt63_powerTimeMeas_set (ViSession instrumentHandle,
                                            ViBoolean powervsTimeMeasMode,
                                            ViInt16 YScale);

ViStatus CVIFUNC avopt63_parameter_setDECT (ViSession instrumentHandle,
                                            ViInt16 triggerSource,
                                            ViBoolean burstSwitch,
                                            ViBoolean trgSlope,
                                            ViInt16 triggerLevel,
                                            ViBoolean extTrgDelay,
                                            ViInt16 fullSlotNumber,
                                            ViInt16 halfSlotNumber,
                                            ViReal64 delayTimes,
                                            ViBoolean freqDeviationBITSequence,
                                            ViBoolean timingJitterRfptoPP);

ViStatus CVIFUNC avopt63_templatePwrTime_editDECT (ViSession instrumentHandle,
                                                   ViInt16 templateEntry,
                                                   ViReal64 y0Lvl,
                                                   ViBoolean y0Unit,
                                                   ViReal64 y1Lvl,
                                                   ViBoolean y1Unit,
                                                   ViReal64 y2Lvl,
                                                   ViBoolean y2Unit,
                                                   ViReal64 y3Lvl,
                                                   ViBoolean y3Unit,
                                                   ViReal64 y4Lvl,
                                                   ViBoolean y4Unit,
                                                   ViReal64 y5Lvl,
                                                   ViBoolean y5Unit);

ViStatus CVIFUNC avopt63_average_setOpt63 (ViSession instrumentHandle,
                                           ViInt16 function, ViInt16 AVGTimes);

ViStatus CVIFUNC avopt63_averaging_setGSM (ViSession instrumentHandle,
                                           ViInt16 function, ViInt16 AVGTimes);

ViStatus CVIFUNC avopt63_averaging_setDECT (ViSession instrumentHandle,
                                             ViInt16 function, ViInt16 AVGTimes);

ViStatus CVIFUNC avopt63_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function);

ViStatus CVIFUNC avopt63_act_DcCal (ViSession instrumentHandle);

ViStatus CVIFUNC avopt63_measStart_actOpt63 (ViSession instrumentHandle,
                                             ViInt16 function);

ViStatus CVIFUNC avopt63_measStart_actGSM (ViSession instrumentHandle,
                                           ViInt16 function);

ViStatus CVIFUNC avopt63_measStart_actDECT (ViSession instrumentHandle,
                                            ViInt16 function);

ViStatus CVIFUNC avopt63_graphType_actPErr (ViSession instrumentHandle,
                                            ViInt16 graphType);

ViStatus CVIFUNC avopt63_graphType_actGSM (ViSession instrumentHandle,
                                           ViInt16 graphType);

ViStatus CVIFUNC avopt63_graphType_actDECT (ViSession instrumentHandle,
                                            ViInt16 graphType, ViInt16 startBit);

ViStatus CVIFUNC avopt63_graphMkr_act (ViSession instrumentHandle,
                                       ViBoolean marker);

          
ViStatus CVIFUNC avopt63_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode);

ViStatus CVIFUNC avopt63_check_modeGSM (ViSession instrumentHandle,
                                        ViInt16 *link, ViInt16 *system,
                                        ViInt16 *modulation);

ViStatus CVIFUNC avopt63_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt63_data_OnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito,
                                          ViInt16 *judgement);



ViStatus CVIFUNC avopt63_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], ViReal64 level[],
                                       ViInt16 judgement[]);

ViStatus CVIFUNC avopt63_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt63_data_OBW (ViSession instrumentHandle, ViReal64 *OBWHz,
                                   ViReal64 *lowFreqHz, ViReal64 *hiFreqHz,
                                   ViInt16 *judgement);



ViStatus CVIFUNC avopt63_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[]);

ViStatus CVIFUNC avopt63_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[]);

ViStatus CVIFUNC avopt63_data_INSPR (ViSession instrumentHandle,
                                     ViInt16 *numberofPoint,
                                     ViReal64 frequencyHz[], ViReal64 leveldBm[],
                                     ViInt16 judge[]);

ViStatus CVIFUNC avopt63_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judgement[]);



ViStatus CVIFUNC avopt63_data_TxPowerDECT (ViSession instrumentHandle,
                                       ViReal64 *powerLeveldBm,
                                       ViReal64 *powerLevelW);

ViStatus CVIFUNC avopt63_data_TxPowerGSM (ViSession instrumentHandle,
                                          ViInt16 *modulation,
                                          ViReal64 *powerLeveldBm,
                                          ViReal64 *powerLevelW,
                                          ViReal64 *peakFactordB);

ViStatus CVIFUNC avopt63_dataPhaseError (ViSession instrumentHandle,
                                          ViReal64 *phaseErrordegree,
                                          ViReal64 *peakPhaseError,
                                          ViReal64 *peakPhaseErrorPosition,
                                          ViReal64 *frequencyError);

ViStatus CVIFUNC avopt63_dataGraphPhaseError (ViSession instrumentHandle,
                                               ViInt16 graphType,
                                               ViInt16 *xDataSize, ViReal64 xData[],
                                               ViInt16 *yDataSize, ViReal64 yData[]);

ViStatus CVIFUNC avopt63_dataDmlPhaseError (ViSession instrumentHandle,
                                            ViInt16 *demodulatedDataSize,
                                            ViInt32 demodulatedData[]);

ViStatus CVIFUNC avopt63_dataModAcc (ViSession instrumentHandle,
                                      ViReal64 *burstAmpDroop,
                                      ViReal64 *crrFreqErr,
                                      ViReal64 *IQOriginOffset,
                                      ViReal64 *magnitudeErr,
                                      ViReal64 *phaseErr,
                                      ViReal64 *errorVectorMag,
                                      ViReal64 *per_95th,
                                      ViReal64 *avgdPeakEVM,
                                      ViReal64 *peakMagErr,
                                      ViInt16 *pos_PeakMagErr,
                                      ViReal64 *peakPhsErr,
                                      ViInt16 *pos_PeakPhsErr,
                                      ViReal64 *peakEVM,
                                      ViInt16 *pos_PeakEVM);

ViStatus CVIFUNC avopt63_dataGraphModAcc (ViSession instrumentHandle,
                                           ViInt16 graphType, 
                                           ViInt16 *XDataSize, ViReal64 XData[], 
                                           ViInt16 *YDataSize, ViReal64 YData[]);

ViStatus CVIFUNC avopt63_dataDmlModAcc (ViSession instrumentHandle,
                                        ViInt16 *demodulatedDataSize,
                                        ViInt32 demodulatedData[]);

ViStatus CVIFUNC avopt63_dataFreqDevRandSeq (ViSession instrumentHandle,
                                              ViReal64 *freqErr,
                                              ViReal64 *maxPeakDev,
                                              ViReal64 *minPeakDev);

ViStatus CVIFUNC avopt63_dataFreqDevSTDSeq (ViSession instrumentHandle,
                                            ViReal64 *frequencyErrorHz,
                                            ViReal64 *syncFieldMaxPeakDevHz,
                                            ViReal64 *syncFieldMinPeakDevHz,
                                            ViReal64 *lpbackFieldMaxPeakDevHz,
                                            ViReal64 *lpbackFieldMinPeakDevHz);


ViStatus CVIFUNC avopt63_dataGraphFreqDeviation (ViSession instrumentHandle,
                                                  ViInt16 graphType,
                                                  ViInt16 *xDataSize, ViReal64 xData[],
                                                  ViInt16 *yDataSize, ViReal64 yData[]);

ViStatus CVIFUNC avopt63_dataDmlFreqDeviation (ViSession instrumentHandle,
                                               ViInt16 *demodulatedDataSize,
                                               ViInt32 demodulatedData[]);

ViStatus CVIFUNC avopt63_data_TimJitDECT (ViSession instrumentHandle,
                                          ViReal64 *maxPeakJitter,
                                          ViReal64 *minPeakJitter,
                                          ViReal64 *averageJitter);

ViStatus CVIFUNC avopt63_data_PwrVsTime (ViSession instrumentHandle,
                                         ViReal64 *leveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt63_close (ViSession instrSession);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
